#! /usr/bin/python
# -*- coding: iso-8859-15 -*-
#
#     file     : projects/fantec/dtv_converter.py
#     author   : Marcus Rickert (marcus.rickert@web.de)
#     RCS      : $Id$
#     purpose  : script to create a new dtv_channel.txt from an existing one
#     version  : 0.9
#
# calling syntax:
#
#     dtv_converter.py [--build] <DTV-CHANNEL-FILE>
#
# to do:
#   - handle character set correctly

import string
import struct
import binascii

from t_AppException import *
from t_OptionHandler import *
from l_crc32 import *
from UserString import MutableString

OPTION_BUILD = "build"

DEFAULT_SOURCE = "dtv_channel.txt"
DEFAULT_TEXT_CONFIG_EXTENSION = ".cfg"
DEFAULT_NEW_DATA_EXTENSION = ".new"
DEFAULT_BUILD = 0

HEADER_SIZE = 8
FOOTER_SIZE = 4
ENTRY_SIZE = 576
CHANNEL_OFFSET = 0
NAME_OFFSET = 8
NAME_SIZE = 32
COLUMN_SEPERATOR = "|"
ACTIVE_CHAR = 'a'
INACTIVE_CHAR = 'i'

# -----------------------------------------------------------------------------------

def evaluate(p_in_filename, p_config_filename):

    v_f_in = open(p_in_filename, 'r')
    configData = v_f_in.read()
    v_f_in.close()
    v_f_config = open(p_config_filename, "w")
    pos = HEADER_SIZE
    count = 0
    
    while pos < len(configData) - FOOTER_SIZE:

        channel = ord(configData[pos+CHANNEL_OFFSET])
        name = string.strip(configData[pos+NAME_OFFSET:pos+NAME_OFFSET+NAME_SIZE], '\0')
        v_f_config.write("%s%s%d%s%s\n" % (ACTIVE_CHAR, COLUMN_SEPERATOR, channel, COLUMN_SEPERATOR, name))
        pos = pos + ENTRY_SIZE
        count = count + 1

    v_f_config.close()
    print "%d channel configuration entries written to %s" % (count, p_config_filename)

# -----------------------------------------------------------------------------------

def build(p_in_filename, p_config_filename, p_out_filename):

    data = MutableString()
    v_f_in = open(p_in_filename, 'r')    
    configData = v_f_in.read()
    v_f_in.close()
    v_f_config = open(p_config_filename, "r")
    v_f_out = open(p_out_filename, "w")
    pos = HEADER_SIZE
    count = 0
    
    for line in v_f_config:

        line = line.rstrip('\n')
        (statusChar, channelString, name) = string.split(line, COLUMN_SEPERATOR)

        if statusChar == ACTIVE_CHAR:
            
            channel= int(channelString)
            entry = configData[HEADER_SIZE + (channel - 1) * ENTRY_SIZE:HEADER_SIZE + channel*ENTRY_SIZE]
            count = count + 1
            
            # set the index, the length of the name and the name itself
            entry = "%s%s%s%s%s" % (struct.pack("i", count), struct.pack("i", len(name)), name, chr(0), entry[2*4 + len(name) + 1:])
            data += entry

    v_f_config.close()
    newConfigData = "%s%s%s" % (configData[0:4], struct.pack("i", count), data)  
    v_f_out.write(newConfigData)
    v_f_out.write(struct.pack("I", GetCRC32(newConfigData)))
    v_f_out.close()
    print "%d channel entries written to %s" % (count, p_out_filename)


# -----------------------------------------------------------------------------------

if __name__ == '__main__':

    try:

        v_Options = [(OPTION_BUILD , "builds the new channel file (instead of evaluating the old one)" )]
        v_OptionHandler = t_OptionHandler(v_Options, 1, 1)
        v_OptionHandler.parse()
        v_DataFilename = v_OptionHandler.f_Args[0]
        v_Build = v_OptionHandler.getParameter(OPTION_BUILD, 0)
        v_ConfigFilename = "%s%s" % (v_DataFilename, DEFAULT_TEXT_CONFIG_EXTENSION)

        if v_Build:

            v_newDataFilename = "%s%s" % (v_DataFilename, DEFAULT_NEW_DATA_EXTENSION)
            build(v_DataFilename, v_ConfigFilename, v_newDataFilename)
            
        else:
            
            evaluate(v_DataFilename, v_ConfigFilename)        
                                    
    except IOError:
        printExceptionMessages()
            
    except t_EnvException:
        printExceptionMessages()
        v_OptionHandler.listOptions();

    except t_AppException:
        printExceptionMessages()
            
    except Exception:
        raise

